@@ -2,59 +2,15 @@
 
 import os
 import os.path
-import platform
 import shutil
 import stat
-import subprocess
 import sys
 import sysconfig
 import tempfile
 
 import pytest
 
-from filelock import FileLock
-
-
-def _build_and_reinstall_build(test_mode):
-    temp = tempfile.mkdtemp()
-    try:
-        subprocess.check_output(
-            [sys.executable, '-m', 'build', f'--{test_mode}', '--no-isolation', '--outdir', temp],
-        )
-        dist_file = next(d for d in os.listdir(temp) if d.endswith('.whl' if test_mode == 'wheel' else '.tar.gz'))
-        subprocess.check_call(
-            [
-                sys.executable,
-                '-m',
-                'pip',
-                'install',
-                '--upgrade',  # ``--upgrade`` will uninstall build prior to installing the ``dist_file``
-                os.path.join(temp, dist_file),
-            ],
-        )
-    finally:
-        shutil.rmtree(temp)
-
-
-def _one_time_setup():
-    test_mode = os.environ.get('TEST_MODE')
-    if not test_mode:
-        return
-
-    if test_mode == 'path':
-        project_root = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
-        sys.path.insert(0, os.path.join(project_root, 'src'))
-    elif test_mode in {'sdist', 'wheel'}:
-        status_marker_file = os.path.join(os.environ['TEST_STATUS_DIR'], 'status-marker')
-        with FileLock(status_marker_file + '.lock'):
-            if not os.path.exists(status_marker_file):
-                _build_and_reinstall_build(test_mode)
-
-                with open(status_marker_file, 'wb'):
-                    pass
-
-
-_one_time_setup()
+import build.env
 
 
 def pytest_addoption(parser):
@@ -64,7 +20,9 @@ def pytest_addoption(parser):
     parser.addoption('--only-integration', action='store_true', help='only run the integration tests')
 
 
-PYPY3_WIN_VENV_BAD = platform.python_implementation() == 'PyPy' and os.name == 'nt'
+PYPY3_WIN_VENV_BAD = (
+    sys.implementation.name == 'pypy' and sys.implementation.version < (7, 3, 9) and sys.platform.startswith('win')
+)
 PYPY3_WIN_M = 'https://foss.heptapod.net/pypy/pypy/-/issues/3323 and https://foss.heptapod.net/pypy/pypy/-/issues/3321'
 
 
@@ -79,6 +37,8 @@ def pytest_collection_modifyitems(config, items):
         return
     for item in items:
         is_integration_file = is_integration(item)
+        if PYPY3_WIN_VENV_BAD and item.get_closest_marker('pypy3323bug') and os.environ.get('PYPY3323BUG', None):
+            item.add_marker(pytest.mark.xfail(reason=PYPY3_WIN_M, strict=False))
         if PYPY3_WIN_VENV_BAD and item.get_closest_marker('isolated'):
             if not (is_integration_file and item.originalname == 'test_build') or (
                 hasattr(item, 'callspec') and '--no-isolation' not in item.callspec.params.get('args', [])
@@ -108,74 +68,20 @@ def packages_path():
     return os.path.realpath(os.path.join(__file__, '..', 'packages'))
 
 
-@pytest.fixture
-def legacy_path(packages_path):
-    return os.path.join(packages_path, 'legacy')
+def generate_package_path_fixture(package_name):
+    @pytest.fixture
+    def fixture(packages_path):
+        return os.path.join(packages_path, package_name)
+
+    return fixture
 
 
-@pytest.fixture
-def test_flit_path(packages_path):
-    return os.path.join(packages_path, 'test-flit')
-
-
-@pytest.fixture
-def test_bad_syntax_path(packages_path):
-    return os.path.join(packages_path, 'test-bad-syntax')
-
-
-@pytest.fixture
-def test_no_backend_path(packages_path):
-    return os.path.join(packages_path, 'test-no-backend')
-
-
-@pytest.fixture
-def test_no_project_path(packages_path):
-    return os.path.join(packages_path, 'test-no-project')
-
-
-@pytest.fixture
-def test_no_requires_path(packages_path):
-    return os.path.join(packages_path, 'test-no-requires')
-
-
-@pytest.fixture
-def test_optional_hooks_path(packages_path):
-    return os.path.join(packages_path, 'test-optional-hooks')
-
-
-@pytest.fixture
-def test_typo(packages_path):
-    return os.path.join(packages_path, 'test-typo')
-
-
-@pytest.fixture
-def test_no_prepare_path(packages_path):
-    return os.path.join(packages_path, 'test-no-prepare')
-
-
-@pytest.fixture
-def test_setuptools_path(packages_path):
-    return os.path.join(packages_path, 'test-setuptools')
-
-
-@pytest.fixture
-def test_bad_wheel_path(packages_path):
-    return os.path.join(packages_path, 'test-bad-wheel')
-
-
-@pytest.fixture
-def test_cant_build_via_sdist_path(packages_path):
-    return os.path.join(packages_path, 'test-cant-build-via-sdist')
-
-
-@pytest.fixture
-def test_invalid_requirements_path(packages_path):
-    return os.path.join(packages_path, 'test-invalid-requirements')
-
-
-@pytest.fixture
-def test_metadata(packages_path):
-    return os.path.join(packages_path, 'test-metadata')
+# Generate path fixtures dynamically.
+package_names = os.listdir(os.path.join(os.path.dirname(__file__), 'packages'))
+for package_name in package_names:
+    normalized_name = package_name.replace('-', '_')
+    fixture_name = f'package_{normalized_name}'
+    globals()[fixture_name] = generate_package_path_fixture(package_name)
 
 
 @pytest.fixture
@@ -198,3 +104,8 @@ def tmp_dir():
     yield path
 
     shutil.rmtree(path)
+
+
+@pytest.fixture(autouse=True)
+def force_venv(mocker):
+    mocker.patch.object(build.env, '_should_use_virtualenv', lambda: False)
